#ifndef LIGHT_H
#define LIGHT_H

/**
 * @file
 * Klassendeklaration fuer OpenGL-Lichter
 *
 * @author
 * Philip Lehmann-Boehm (minf8180)
 */

#include "types.h"

#include <GL/gl.h>
class Light {

	private:
		// Die maximale Anzahl an Lichtquellen
		static const int maxLights = 8;

		// Array zur Verwaltung der schon benutzten Lichtquellen
		static GLenum usedLights[8];

		// Index des naechsen freien Platzes im Licht-Verwaltungsarray
		static int nextFreeLight;

	protected:
		// Die zugehoerige OpenGL-Lichtquelle.
		GLenum glLight;

	public:
		/**
		 * Konstruktor fuer das abstrakte Licht.
		 * @param cAmb Der ambiente Lichtanteil
		 * @param cDiff Der diffuse Lichtanteil
		 * @param cSpec Der spekulare Lichtanteil
		 */
		Light(CGColor4f cAmb, CGColor4f cDiff, CGColor4f cSpec);

		/**
		 * Destruktor
		 */
		~Light();

		/**
		 * Schaltet die Lichtquelle ein.
		 */
		void on();

		/**
		 * Schaltet die Lichtquelle aus.
		 */
		void off();
};

class DirectedLight : public Light {
	private:
		// Die Leuchtrichtung
		CGPoint4Df dir;
	public:

		/**
		 * Konstruktor
		 * @param cAmb Der ambiente Lichtanteil
		 * @param cDiff Der diffuse Lichtanteil
		 * @param cSpec Der spekulare Lichtanteil
		 * @param dir Die Leuchtrichtung
		 */
		DirectedLight(CGColor4f cAmb, CGColor4f cDiff, CGColor4f cSpec, CGPoint3Df dir);

		/**
		 * Schaltet die Lichtquelle ein.
		 */
		void on();
};

class PositionedLight : public Light {
	protected:
		// Die Position der Lichtquelle
		CGPoint4Df pos;
	public:

		/**
		 * Konstruktor
		 * @param cAmb Der ambiente Lichtanteil
		 * @param cDiff Der diffuse Lichtanteil
		 * @param cSpec Der spekulare Lichtanteil
		 * @param pos Die Lichtposition
		 */
		PositionedLight(CGColor4f cAmb, CGColor4f cDiff, CGColor4f cSpec, CGPoint3Df pos);

		/**
		 * Setzt die Abschwaechung der Lichtquelle
		 * @param constant Der konstante Abschwaechungsfaktor
		 * @param linear Der lineare Abschwaechungsfaktor
		 * @param quadratic Der quadratische Abschwaechungsfaktor
		 */
		void setAttenuation(GLfloat constant, GLfloat linear, GLfloat quadratic);

		/**
		 * Schaltet die Lichtquelle ein.
		 */
		void on();
};

class SpotLight : public PositionedLight {
	private:
		// Die Leuchtrichtung
		CGPoint3Df dir;
	public:

		/**
		 * Konstruktor
		 * @param cAmb Der ambiente Lichtanteil
		 * @param cDiff Der diffuse Lichtanteil
		 * @param cSpec Der spekulare Lichtanteil
		 * @param pos Die Lichtposition
		 * @param dir Die Leuchtrichtung
		 * @param exponent Die Fokussierung des Spots
		 * @param cutoff Der Austrittswinkel des Spots
		 */
		SpotLight(CGColor4f cAmb, CGColor4f cDiff, CGColor4f cSpec, CGPoint3Df pos, CGPoint3Df dir, GLfloat exponent, GLfloat cutOff);

		/**
		 * Schaltet die Lichtquelle ein.
		 */
		void on();
};

// Fehler, der geworfen wird, wenn die maximale Anzahl an Lichtern erreicht ist.
class LightError {};

#endif
