/**
 * @file
 * Klassenimplementierung fuer die Logikklasse
 *
 * @author
 * Christian Harendt (minf2964)
 * Philip Lehmann-Boehm (minf8180)
 */

#include "Logic.h"
#include <QLinkedList>
#include <QLinkedListIterator>
#include <QMutableLinkedListIterator>

#include <QDebug>
#include <iostream>
#include <math.h>

using namespace std;

/**
 * Konstruktor
 */
Logic::Logic() : QObject() {

	// Kramszeug initialisieren
	rotation = 0.0;
	timeSinceLastRequest = 0;
	blockSize = 0;
	personCounta = personCountb = 0;
	firstRead = true;
	tte = 0.0;
	cacheSwaped = false;

	// Streams initialisieren
	out = new QDataStream(&block, QIODevice::WriteOnly);
	out->setVersion(QDataStream::Qt_4_0);
    in = new QDataStream(&socket);
    in->setVersion(QDataStream::Qt_4_0);

	// Ereignis, wenn Daten vom Server ankommen, mit der Verarbeitungsroutine verknuepfen
	connect(&socket, SIGNAL(readyRead()), this, SLOT(readData()));
}

/**
 * Destruktor
 */
Logic::~Logic() {
	delete out;
	delete in;
	radarlist.clear();
}

/**
 * Berechnet den naechsten Schritt
 * @param interval Vergangene Zeit seit dem letzten Schritt
 * @return Die Schrittberechnung hat geklappt
 */
bool Logic::nextStep(double interval) {
	timeSinceLastRequest += interval;
	// Nur wenn die konstante Zeit vergangen ist, einen Request an den Server schicken
	if (timeSinceLastRequest > serverRequestPause) {
		if (socket.state() != QAbstractSocket::ConnectedState) {
			return false;
		}
		timeSinceLastRequest = 0;
		block[0] = cmdGetPersons;
//		*out << cmdGetPersons;
		socket.write(block);
		socket.flush();

	}

	// Rotation weitersetzen
	rotation += 80.0 * interval;
	normalizeAngle(&rotation);
	return true;
}

/**
 * Liefert den momentanen Stand der Rotation
 * @return Der momentane Stand der Rotation
 */
double Logic::getRotation() {
	return rotation;
}

/**
 * Normalisiert einen Winkel in den Grenzen 0-360 Grad
 * @param angle Der zu normalisierende Winkel
 */
void Logic::normalizeAngle(double *angle) {
	while (*angle < 0.0) {
		*angle += 360.0;
	}
	while (*angle > 360.0) {
		*angle -= 360.0;
	}
}

/**
 * Baut die Verbindung zum Server auf.
 * @param host Die Serveradresse
 * @param port Der Verbindungsport
 * @return Ob die Verbindung geklappt hat oder nicht
 */
bool Logic::connectToHost(QString host, int port) {
	socket.connectToHost(host, port);
	return socket.waitForConnected(connectTimeout);
}

/**
 * Liest die eingehenden Serverdaten aus
 */
void Logic::readData() {

	// Warten, bis der int mit der Datengroesse da ist.
	if (blockSize == 0) {
		if (socket.bytesAvailable() < (int)sizeof(quint16))
			return;
		*in >> blockSize;
	}

	// Warten, bis alle Daten geschickt wurden.
	if (socket.bytesAvailable() < blockSize) {
		return;
	}

	blockSize = 0;

	// Daten auslesen und die Personen ins Array eintragen
	quint16 count;
    *in >> count;
    personCount = count;
    for (int i = 0; i < personCount; ++i) {
		*in >> persons[i].x;
		*in >> persons[i].y;
		*in >> persons[i].size;
	}

    if (firstRead) {
        personCounta = personCount;
        for (int i = 0; i < count; ++i) {
            slowpersonstatea[i].x = persons[i].x;
            slowpersonstatea[i].y = persons[i].y;
            slowpersonstatea[i].hit = false;
        }

        personCountb = personCount;
        for (int i = 0; i < count; ++i) {
            slowpersonstateb[i].x = persons[i].x;
            slowpersonstateb[i].y = persons[i].y;
            slowpersonstateb[i].hit = false;
        }
    }

    firstRead = false;
}

void Logic::buildState() {
    cacheSwaped = !cacheSwaped;

    if (cacheSwaped) {
        personCounta = personCount;
        for (int i = 0; i < personCounta; ++i) {
            slowpersonstatea[i].x = persons[i].x;
            slowpersonstatea[i].y = persons[i].y;
            slowpersonstatea[i].hit = false;
        }

    } else {
        personCountb = personCount;
        for (int i = 0; i < personCountb; ++i) {
            slowpersonstateb[i].x = persons[i].x;
            slowpersonstateb[i].y = persons[i].y;
            slowpersonstateb[i].hit = false;
        }
    }
}

/**
 * Liefert die Anzahl der Personen
 * @return Die Anzahl der Personen
 */
int Logic::getPersonCount() {
    if (cacheSwaped) {
        return personCounta;
    } else {
        return personCountb;
    }
}

/**
 * Liefert eine bestimmte Person
 * @param i Der Index der gewuenschten Person
 * @return Die gewuenschte Person
 */
CachePerson* Logic::getPerson(int i) {
	if (cacheSwaped) {
        return &(slowpersonstatea[i]);
    } else {
        return &(slowpersonstateb[i]);
    }
}

/**
 * Updated den Radarcache wenn noetig
 * @param interval Vergangene Zeit seit dem letzten Schritt
 */
void Logic::update(double interval) {
    tte -= interval;

    if (tte <= 0.0) {
        tte = 0.7;
        buildState();
        cout << "build" << endl;
    }

    // Pruefen ob Radarstrahl eine Person ueberfaehrt
    float x, y;

    for (int i = 0; i < getPersonCount(); ++i) {

        CachePerson *rp = getPerson(i);


        if (rp->hit) break;

        x = (rp->x * 3.0f - 1.5f);
        y = (rp->y * 3.0f - 1.5f);


        float alpha = atan( x / y );

        alpha *= 180.0 / 3.14159; // rad->deg

        // Q1
        if ( x >= 0.0 && y < 0.0)
        {
            alpha *= -1.0;
        }
        // Q2
        else if (x >= 0.0 && y >= 0.0)
        {
            alpha = (90.0 - alpha) + 90.0;
        }
        // Q3
        else if (x < 0.0 && y >= 0.0)
        {
            alpha = -alpha + 180.0;
        }
        // Q4
        else
        {
            alpha = (90.0 - alpha) + 270.0;
        }

        //if (i == 2) cout << "i: " << i << " winkel: " << alpha << endl;

        /*if (fabs(alpha - getRotation()) <= 5.0)
        {
            slowpersonstate[i].hit = true;
            cout << "hit @ green: " << getRotation() << " alpha: " << alpha << endl;

            RadarPerson e;
            e.x = x;
            e.y = y;
            e.ttl = 2.0;
            radarlist = addElem(e, radarlist);
        }*/

        if (fabs(alpha - getRotation()) <= 5.0)
        {
            rp->hit = true;
            cout << "hit @ green: " << getRotation() << " alpha: " << alpha << endl;

            RadarPerson e;
            e.x = x;
            e.y = y;
            e.ttl = 2.0;
            radarlist << e;
        }
    }

    //liste updaten, unsichtbare rausschmeissen, ttl updaten
    /*List rl = radarlist;
    for (unsigned i=0; i<length(rl); ++i) {
        if (isEmptyList(rl)) break;

        rl->info.ttl -= interval;

        if (rl->info.ttl <= 0.0)
          rl = removeElem(rl->info, rl);

        rl = tail(rl);
    }

    cout << "ls: " << length(radarlist) << endl;*/

    if (!radarlist.empty()) {
        RadarPerson e = radarlist.first();

        QLinkedListIterator<RadarPerson> it(radarlist);

        e.ttl -= interval;
        if (e.ttl <= 0.0)
            radarlist.removeAll(e);

         while (it.hasNext())  {
            e = it.next();
            if (e.ttl - interval > 0.0) {
                radarlist.removeAll(e);
                e.ttl -= interval;
                radarlist << e;
            } else {
                radarlist.removeAll(e);
            }
        }
    }
}

/**
 * Liefert die Liste mit den Personen auf dem Radar
 */
QLinkedList<RadarPerson>* Logic::getList() {
    return &radarlist;
}
