/**
 * @file
 * Klassenimplementierung fuer die Einstellungen
 *
 * @author
 * Philip Lehmann-Boehm (minf8180)
 */

#include "Settings.h"

#include <QFile>

/**
 * Konstruktor
 * @param filename Der Dateiname der Einstellungen
 */
Settings::Settings(QString filename) {
		QDomDocument doc("settings");
		// Datei oeffnen
		QFile settingsFile(filename);
		if (!settingsFile.open(QIODevice::ReadOnly)) {
			throw new SettingsError;
		}
		// Datei einlesen
		if (!doc.setContent(&settingsFile)) {
			settingsFile.close();
			throw new SettingsError;
		}
		settingsFile.close();
		settings = doc;
		this->filename = filename;
}

/**
 * Liefert eine Int-Einstellung
 * @param name Der Name der Einstellung
 * @param defValue Default-Wert, wenn die Einstellung nicht in der Datei ist
 * @return Die Einstellung
 */
int Settings::get(QString name, int defValue) {
	// Zum Einstellungsknoten navigieren.
	QDomElement e = settings.documentElement().firstChildElement(name);
	if (e.isNull()) {
		return defValue;
	}
	// Die Einstellung als Int lesen.
	bool ok;
	int result = e.text().toInt(&ok);
	if (!ok) {
		return defValue;
	}
	return result;
}

/**
 * Liefert eine Float-Einstellung
 * @param name Der Name der Einstellung
 * @param defValue Default-Wert, wenn die Einstellung nicht in der Datei ist
 * @return Die Einstellung
 */
float Settings::get(QString name, float defValue) {
	// Zum Einstellungsknoten navigieren.
	QDomElement e = settings.documentElement().firstChildElement(name);
	if (e.isNull()) {
		return defValue;
	}
	// Die Einstellung als Float lesen.
	bool ok;
	float result = e.text().toFloat(&ok);
	if (!ok) {
		return defValue;
	}
	return result;
}

/**
 * Liefert eine String-Einstellung
 * @param name Der Name der Einstellung
 * @param defValue Default-Wert, wenn die Einstellung nicht in der Datei ist
 * @return Die Einstellung
 */
QString Settings::get(QString name, QString defValue) {
	// Zum Einstellungsknoten navigieren.
	QDomElement e = settings.documentElement().firstChildElement(name);
	if (e.isNull()) {
		return defValue;
	}
	return e.text();
}

/**
 * Aendert eine String-Einstellung
 * @param name Der Name der Einstellung
 * @param value Der neue Wert der Einstellung
 */
void Settings::change(QString name, QString value) {

	// Das neue Element im Baum zusammenstellen.
	QDomElement newElement = settings.createElement(name);
	QDomText newText = settings.createTextNode(value);
	newElement.appendChild(newText);

	// Je nachdem, ob eine Einstellung mit dem Namen im Baum ist, entweder den alten
	// Knoten ersetzen oder den neuen einfach anhaengen.
	QDomElement e = settings.documentElement().firstChildElement(name);
	if (e.isNull()) {
		settings.documentElement().appendChild(newElement);
	} else {
		QDomElement oldElement = settings.documentElement().firstChildElement(name);
		settings.documentElement().replaceChild(newElement, oldElement);
	}
}

/**
 * Aendert eine Int-Einstellung
 * @param name Der Name der Einstellung
 * @param value Der neue Wert der Einstellung
 */
void Settings::change(QString name, int value) {
	// In String umwandeln und die entsprechende change-Methode aufrufen.
	QString strValue;
	strValue.setNum(value);
	change(name, strValue);
}

/**
 * Aendert eine Float-Einstellung
 * @param name Der Name der Einstellung
 * @param value Der neue Wert der Einstellung
 */
void Settings::change(QString name, float value) {
	// In String umwandeln und die entsprechende change-Methode aufrufen.
	QString strValue;
	strValue.setNum(value);
	change(name, strValue);
}

/**
 * Schreibt die Einstellungen zurueck in die Datei
 */
void Settings::write() {
	QFile data(filename);
 	if (data.open(QFile::WriteOnly | QFile::Truncate)) {
    	QTextStream out(&data);
    	settings.save(out, 4);
	} else {
		throw new SettingsError;
	}
}
